/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_connection_progress.c,v $
 *
 * $Author: mr $
 * $Date: 2004/02/01 20:55:39 $
 * 
 * $Revision: 1.5 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <glib.h>

#ifndef G_OS_WIN32
#include <libgnome/gnome-i18n.h>
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gj_main.h"
#include "gj_parser.h"
#include "gj_iq.h"
#include "gj_iq_requests.h"
#include "gj_roster.h"
#include "gj_translate.h"
#include "gj_me.h"
#include "gj_support.h"
#include "gj_config.h"
#include "gj_stream.h"
#include "gj_connection.h"

#include "gj_gtk_roster.h"
#include "gj_gtk_connection_progress.h"
#include "gj_gtk_connection_settings.h"
#include "gj_gtk_password_required.h"
#include "gj_gtk_support.h"


/* local structs */
enum _GjConnectionProgressState 
  {
    GjConnectionProgressStateUninitiated = 1,
    GjConnectionProgressStateIdle,
    GjConnectionProgressStateConnecting,
    GjConnectionProgressStateAuthenticationMethodRequested,
    GjConnectionProgressStateAuthenticating,
    GjConnectionProgressStateReady,
    GjConnectionProgressStateDisconnected,
    
    GjConnectionProgressStateEnd
  }; 


typedef struct 
{
  GtkWidget *window;
  
  GtkWidget *button;

  GtkWidget *label_desc1;
  GtkWidget *label_desc2;

  GtkWidget *image;

  GtkWidget *progressbar;

  gint state;

  gchar *server;
  gint16 port;
  gchar *username;
  gchar *password;
  gchar *resource;

  GjStream stream;

  GjConnectionProgressCallback cb_connected;
  GjConnectionProgressCallback cb_disconnected;

  gboolean is_disconnect_planned;

  gint timeout_id;

} GjConnectionProgressWindow;


/* local functions */
static gboolean gj_gtk_cp_set_state(GjConnectionProgressWindow *window, gint state);

static void gj_gtk_cp_request_auth_method_cb(GjInfoquery iq, GjInfoquery iq_related, gpointer user_data);
static void gj_gtk_cp_request_auth_cb(GjInfoquery iq, GjInfoquery iq_related, gpointer user_data);

/* 
 * states
 */

/* translations */
static const gchar *gj_gtk_cp_translate_state(gint state);

/* state machine */
static void gj_gtk_cp_state_machine();
static void gj_gtk_cp_state_handle_uninitiated();
static void gj_gtk_cp_state_handle_idle();
static void gj_gtk_cp_state_handle_connecting();
static void gj_gtk_cp_state_handle_auth_method_requested();
static void gj_gtk_cp_state_handle_authenticating();
static void gj_gtk_cp_state_handle_ready();
static void gj_gtk_cp_state_handle_disconnected();

static void gj_gtk_cp_connection_start_cb(GjStream stm, gint event_id);
static void gj_gtk_cp_connection_stop_cb(GjStream stm, gint event_id);
static void gj_gtk_cp_connection_error_cb(GjStream stm, gint event_id);

static void gj_gtk_cp_complete(gboolean success);

static gboolean gj_gtk_cp_progress_timeout_cb(GjConnectionProgressWindow *window);
static void gj_gtk_cp_enter_password_cb(gboolean success, const gchar *password);

/* gui callbacks */
static void on_button_action_clicked(GtkButton *button, GjConnectionProgressWindow *window);
static void on_realize(GtkWidget *widget, GjConnectionProgressWindow *window);
static void on_destroy(GtkWidget *widget, GjConnectionProgressWindow *window);
static gboolean on_delete_event(GtkWidget *widget, GdkEvent *event, GjConnectionProgressWindow *window);


static GjConnectionProgressWindow *current_window = NULL;


gboolean gj_gtk_cp_load(GjConnectionProgressCallback cb_connected,
			GjConnectionProgressCallback cb_disconnected)
{
  GladeXML *xml = NULL;
  GjConnectionProgressWindow *window = NULL;

  if(current_window != NULL)
    {
      gtk_window_present(GTK_WINDOW(current_window->window));
      return TRUE;
    }

  window = g_new0(GjConnectionProgressWindow, 1);

  window->cb_connected = cb_connected;
  window->cb_disconnected = cb_disconnected;

  window->server = g_strdup(gj_config_get_cs_server());
  window->port = gj_config_get_cs_port();
  window->username = g_strdup(gj_config_get_cs_username());
  window->password = g_strdup(gj_config_get_cs_password());
  window->resource = g_strdup(gj_config_get_cs_resource());

  window->is_disconnect_planned = FALSE;

  /* get started ! */
  gj_gtk_cp_set_state(window, GjConnectionProgressStateIdle);
  
  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "connection_progress",
			      NULL,
			      "connection_progress", &window->window,
 			      "button", &window->button,
 			      "label_progress1", &window->label_desc1,
 			      "label_progress2", &window->label_desc2,
 			      "image", &window->image,
 			      "progressbar", &window->progressbar,
			      NULL);

  gj_gtk_glade_connect(xml,
		       window,
		       "connection_progress", "delete_event", on_delete_event,
		       "connection_progress", "destroy", on_destroy,
		       "connection_progress", "realize", on_realize,
		       "button", "clicked", on_button_action_clicked,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  /* remember */
  current_window = window;

  if(gj_config_get_cs_save_password() == FALSE)
    {
      /* get password */
      gj_gtk_pr_load(FALSE, NULL, gj_gtk_cp_enter_password_cb);
    }
  else
    {
      g_message("%s: auto connecting...", __FUNCTION__);
      gtk_widget_show_all(window->window);
      gj_gtk_cp_state_machine(0,0);
    }

  return TRUE;
} 

static void gj_gtk_cp_enter_password_cb(gboolean success, const gchar *password)
{
  /* this could be because:
     - user closed the dialog
     - user clicked ok, and the password wrong,
     - user clicked cancel     */

  if(!current_window)
    return;

  if(success == FALSE)
    {
      g_message("%s: password dialog cancelled, not progressing further.", __FUNCTION__);
      gtk_widget_destroy(current_window->window);
      return;
    }

  /* set password */
  if(current_window->password != NULL)
    {
      g_free(current_window->password);
      current_window->password = NULL;
    }

  current_window->password = g_strdup(password);

  /* start the state machine */
  gtk_widget_show_all(current_window->window);
  gj_gtk_cp_state_machine(0, 0);
}

/* sets */
static gboolean gj_gtk_cp_set_state(GjConnectionProgressWindow *window, gint state)
{
  if(!window)
    return FALSE;

  if(state < 1 || state >= GjConnectionProgressStateEnd)
    {
      g_warning("%s: state:%d was out of range", __FUNCTION__, state);
      return FALSE;
    }

  g_message("+++++++++++++++++++++++++++++++++++++++++");
  g_message("++++ set state:%d->'%s'", state, gj_gtk_cp_translate_state(state));
  g_message("+++++++++++++++++++++++++++++++++++++++++");

  window->state = state;
  return TRUE;
}

static const gchar *gj_gtk_cp_translate_state(gint state)
{
  switch(state)
    {
    case GjConnectionProgressStateUninitiated: return _("uninitiated");
    case GjConnectionProgressStateIdle: return _("idle");
    case GjConnectionProgressStateConnecting: return _("connecting");
    case GjConnectionProgressStateAuthenticationMethodRequested: return _("authentication method requested");
    case GjConnectionProgressStateAuthenticating: return _("authenticating");
    case GjConnectionProgressStateReady: return _("ready");
    case GjConnectionProgressStateDisconnected: return _("disconnected");
    }  

  return "unknown";
}

static void gj_gtk_cp_state_machine(guint event_id, guint related_event_id)
{
  if(!current_window)
    return;

  g_message("*****************************************");
  g_message("**** entered state:%d->'%s'", 
	    current_window->state, 
	    gj_gtk_cp_translate_state(current_window->state));
  g_message("*****************************************");

  switch(current_window->state)
    {
    case GjConnectionProgressStateUninitiated:
      gj_gtk_cp_state_handle_uninitiated(); break;
    case GjConnectionProgressStateIdle:
      gj_gtk_cp_state_handle_idle(); break;
    case GjConnectionProgressStateConnecting:
      gj_gtk_cp_state_handle_connecting(); break;
    case GjConnectionProgressStateAuthenticationMethodRequested: 
/*       gj_gtk_cp_state_handle_auth_method_requested(); break; */
      break;
    case GjConnectionProgressStateAuthenticating:
      gj_gtk_cp_state_handle_authenticating(); break;
    case GjConnectionProgressStateReady:
      gj_gtk_cp_state_handle_ready(); break;
    case GjConnectionProgressStateDisconnected:
      gj_gtk_cp_state_handle_disconnected(); break;
    }  
}

static void gj_gtk_cp_state_handle_uninitiated()
{
  if(!current_window)
    return;
}

static void gj_gtk_cp_state_handle_idle()
{
  GjConnectionProgressWindow *window = NULL;
  GjStream stm = NULL;

  window = current_window;
  if(!window)
    return;

  gj_me_set_is_connected(FALSE);
  gj_me_set_uptime_stop();

  /* set up progress bar */
  gtk_image_set_from_stock(GTK_IMAGE(window->image),GTK_STOCK_DIALOG_INFO,GTK_ICON_SIZE_DIALOG);
  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(window->progressbar),(gdouble)1/4);
  gj_gtk_set_label_with_args(window->label_desc1, "<b>%s</b>", _("Connecting..."));
  gj_gtk_set_label_with_args(window->label_desc2, _("server '%s' on port %d"),
			     window->server, 
			     window->port);

  /* create stream */
  if((stm = gj_stream_new(window->server)) == NULL)
    {
      gtk_image_set_from_stock(GTK_IMAGE(window->image),GTK_STOCK_DIALOG_ERROR,GTK_ICON_SIZE_DIALOG);
      gj_gtk_set_label_with_args(window->label_desc1, "<b>%s</b>", _("Error"));
      gj_gtk_set_label_with_args(window->label_desc2, _("Could not create new stream."));

      g_warning("%s: could not create new stream", __FUNCTION__);

      /* finish */
      gj_gtk_cp_complete(FALSE);

      return;
    }
 
  /* set struct info */
  gj_stream_set_port(stm, window->port);
  gj_stream_set_connected_cb(stm, gj_gtk_cp_connection_start_cb);
  gj_stream_set_disconnected_cb(stm, gj_gtk_cp_connection_stop_cb);
  gj_stream_set_error_cb(stm, gj_gtk_cp_connection_error_cb);

  if(gj_stream_build(stm) == FALSE)
    {
      gtk_image_set_from_stock(GTK_IMAGE(window->image),GTK_STOCK_DIALOG_ERROR,GTK_ICON_SIZE_DIALOG);
      gj_gtk_set_label_with_args(window->label_desc1, "<b>%s</b>", _("Error"));
      gj_gtk_set_label_with_args(window->label_desc2, _("Could not build stream."));

      g_warning("%s: could not build new stream.", __FUNCTION__);

      /* finish */
      gj_gtk_cp_complete(FALSE);
      return;
    }

  if(gj_stream_open(stm) == FALSE)
    {
      gtk_image_set_from_stock(GTK_IMAGE(window->image),GTK_STOCK_DIALOG_ERROR,GTK_ICON_SIZE_DIALOG);
      gj_gtk_set_label_with_args(window->label_desc1, "<b>%s..</b>", _("Connection Error"));
      gj_gtk_set_label_with_args(window->label_desc2, _("Unable to open stream."));

      /* finish */
      gj_gtk_cp_complete(FALSE);
      return;
    }

  /* add timeout for pulsing */
  window->timeout_id = gtk_timeout_add(25, 
					       (GSourceFunc)gj_gtk_cp_progress_timeout_cb, 
					       window);

  /* remember stream */
  window->stream = stm;

  /* set state */
  gj_gtk_cp_set_state(window, GjConnectionProgressStateConnecting);
}

static void gj_gtk_cp_state_handle_connecting(guint event_id, guint related_event_id)
{
  GjConnectionProgressWindow *window = NULL;

  gchar *detail = NULL;
  gchar *jid = NULL;

  window = current_window;
  if(!window)
    return;

  /* create jid */
  jid = g_strdup_printf("%s@%s%s%s", 
			window->username, 
			window->server, 
			window->resource?"/":"",
			window->resource?window->resource:"");

  /* set the fact that we are now connected */
  gj_me_set_is_connected(TRUE);
  gj_me_set_uptime_start();

  gj_me_set_connected_host(window->server);
  gj_me_set_connected_port(window->port);
  gj_me_set_connected_jid(jid);

  gtk_image_set_from_stock(GTK_IMAGE(window->image),GTK_STOCK_DIALOG_INFO,GTK_ICON_SIZE_DIALOG);
  gj_gtk_set_label_with_args(window->label_desc1, "<b>%s...</b>", _("Requesting Authentication Method"));
  gj_gtk_set_label_with_args(window->label_desc2, _("username '%s'"), window->username);
  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(window->progressbar),(gdouble)2/4);
  g_message("%s: response received, event_id:%d", __FUNCTION__, event_id);

  /* clean up */
  if(detail != NULL) g_free(detail);
  if(jid != NULL) g_free(jid);

  /* NOTE: no need to free the iq by event id because there was no iq
     created to generate this response! */

  /* set state */
  gj_gtk_cp_set_state(window, GjConnectionProgressStateAuthenticationMethodRequested);

  /* next request */
  gj_iq_request_auth_method(window->username, gj_gtk_cp_request_auth_method_cb, NULL);
}

static void gj_gtk_cp_request_auth_method_cb(GjInfoquery iq, GjInfoquery iq_related, gpointer user_data)
{
  GjConnectionProgressWindow *window = NULL;

  GjConnection c = NULL;
  GjStream stm = NULL;

  gboolean can_use_digest = TRUE;

  g_message("%s: received available authentication methods", __FUNCTION__);

  if(!iq && !iq_related)
    return;

  window = current_window;
  if(!window)
    return;

  c = gj_iq_get_connection(iq);
  stm = gj_stream_find_by_connection(c);

  if(!stm)
    {
      gtk_image_set_from_stock(GTK_IMAGE(window->image),GTK_STOCK_DIALOG_ERROR,GTK_ICON_SIZE_DIALOG);
      gj_gtk_set_label_with_args(window->label_desc1, "<b>%s</b>", 
				 _("Request for Authentication Method Failed."));
      gj_gtk_set_label_with_args(window->label_desc2, _("Internal Error."));

      g_warning("%s: failed to continue with response to authorisation, "
		"could not find stream for connection to '%s', port:%d", 
		__FUNCTION__, gj_connection_get_host(c), gj_connection_get_port(c));

      /* set state */
      gj_gtk_cp_set_state(window, GjConnectionProgressStateIdle);

      /* finish */
      gj_gtk_cp_complete(FALSE);
      return;
    }

  /* should get if digest method is available here... but we dont yet */
  gj_gtk_cp_state_handle_auth_method_requested(can_use_digest, gj_stream_get_id(stm));
}

static void gj_gtk_cp_state_handle_auth_method_requested(gboolean can_use_digest, const gchar *session_id)
{
  GjConnectionProgressWindow *window = NULL;

  gchar *detail = NULL;
  
  const gchar *password_plain = NULL;
  gchar *password_digest = NULL;

  window = current_window;
  if(!window)
    return;

  detail = g_strdup_printf(_("username '%s'"), window->username);

  gtk_image_set_from_stock(GTK_IMAGE(window->image), GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
  gj_gtk_set_label_with_args(window->label_desc1, "<b>%s</b>", 
			     _("Requesting Authentication..."));
  gj_gtk_set_label_with_args(window->label_desc2, detail);
  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(window->progressbar),(gdouble)3/4);

  /* clean up */
  g_free(detail);

  /* next request */
  if(!session_id || can_use_digest || gj_config_get_cs_use_plain_text_password() == TRUE)
    {
      /* use plain text */
      gj_iq_request_auth_with_plain_password(window->username,
					     window->password,
					     window->resource,
					     gj_gtk_cp_request_auth_cb,
					     NULL);
    }
  else
    {
      /* use digest (SHA1) */
      password_plain = window->password;
      password_digest = gj_get_sha_by_session_id_and_password(session_id, password_plain);

      gj_iq_request_auth_with_digest_password(window->username,
					      password_digest,
					      window->resource,
					      gj_gtk_cp_request_auth_cb,
					      NULL);
      
      if(password_digest != NULL) g_free(password_digest);
    }

  /* set state */
  gj_gtk_cp_set_state(window, GjConnectionProgressStateAuthenticating);
}

static void gj_gtk_cp_request_auth_cb(GjInfoquery iq, GjInfoquery iq_related, gpointer user_data)
{
  GjConnectionProgressWindow *window = NULL;

  xmlNodePtr node = NULL;
  xmlChar *type = NULL;

  g_message("%s: received authentication response", __FUNCTION__);

  if(!iq || !iq_related)
    return;

  window = current_window;
  if(!window)
    return;

  node = gj_iq_get_xml(iq_related);
  type = gj_parser_find_by_attr_and_ret(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      gchar *detail = NULL;

      xmlChar *errorCode = NULL;
      xmlChar *errorMessage = NULL;

      if((node = gj_parser_find_by_node(node, (guchar*)"error")) == NULL)
	{
	  g_warning("%s: failed to find node 'error'", __FUNCTION__);

	  /* set state */
	  gj_gtk_cp_set_state(window, GjConnectionProgressStateIdle);

	  /* finish */
	  gj_gtk_cp_complete(FALSE);

	  return;
	}

      if((errorCode = gj_parser_find_by_attr_and_ret(node, (guchar*)"code")) != NULL)
	detail = g_strdup_printf("%s:%s", _("Error"), errorCode);

      if((errorMessage = gj_parser_find_by_node_and_ret(node, (guchar*)"error")) != NULL) 
	if(detail != NULL) detail = g_strdup_printf("%s - %s",detail,errorMessage);

      gtk_image_set_from_stock(GTK_IMAGE(window->image),GTK_STOCK_DIALOG_ERROR,GTK_ICON_SIZE_DIALOG);
      gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(window->progressbar),(gdouble)4/4);
      gj_gtk_set_label_with_args(window->label_desc1, "<b>%s</b>", _("Failed to authenticate."));
      gj_gtk_set_label_with_args(window->label_desc2, detail);
      if(detail != NULL) g_free(detail);

      /* set state */
      gj_gtk_cp_set_state(window, GjConnectionProgressStateIdle);

      /* finish */
      gj_gtk_cp_complete(FALSE);

      return;
    }

  gtk_image_set_from_stock(GTK_IMAGE(window->image),GTK_STOCK_DIALOG_INFO,GTK_ICON_SIZE_DIALOG);
  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(window->progressbar),(gdouble)4/4);
  gj_gtk_set_label_with_args(window->label_desc1, "<b>%s</b>", _("Authentication successful."));
  gj_gtk_set_label_with_args(window->label_desc2, _("Cleaning up..."));
  
  /* set state */
  gj_gtk_cp_set_state(window, GjConnectionProgressStateReady);

  /* finish */
  gj_gtk_cp_complete(TRUE);  
}

static void gj_gtk_cp_state_handle_authenticating()
{
}

static void gj_gtk_cp_state_handle_ready()
{
}

static void gj_gtk_cp_state_handle_disconnected()
{
  GjConnectionProgressWindow *window = NULL;
  GjConnectionProgressCallback callback = NULL;

  window = current_window;
  if(!window)
    return;

  /* set state */
  gj_me_set_is_connected(FALSE);
  gj_me_set_uptime_stop();

  /* call callback */
  if((callback = window->cb_disconnected) != NULL)
    (callback)();

  /* should we reconnect ? */
  if(window->is_disconnect_planned)
    {
      gtk_widget_destroy(window->window);
      return;
    }
     
  if(gj_config_get_cs_auto_reconnect_on_disconnect() == TRUE && gj_config_get_cs_save_password() == TRUE)
    {
      GjConnectionProgressCallback connected_cb = NULL;
      GjConnectionProgressCallback disconnected_cb = NULL;

      connected_cb = window->cb_connected;
      disconnected_cb = window->cb_disconnected;

      gtk_widget_destroy(window->window);

      gj_gtk_cp_load(connected_cb, disconnected_cb);
    }
  else
    gtk_widget_destroy(window->window);
}

static void gj_gtk_cp_connection_start_cb(GjStream stm, gint event_id)
{
  gj_gtk_cp_state_machine(event_id, 0);
}

static void gj_gtk_cp_connection_stop_cb(GjStream stm, gint event_id)
{
  GjConnectionProgressWindow *window = NULL;

  window = current_window;
  if(!window)
    return;

  if(gj_stream_get_is_disconnect_planned(stm) == TRUE)
    window->is_disconnect_planned = TRUE;

  gj_gtk_cp_set_state(window, GjConnectionProgressStateDisconnected);
  gj_gtk_cp_state_machine(event_id, 0);
}

static void gj_gtk_cp_connection_error_cb(GjStream stm, gint event_id)
{
  GjConnectionProgressWindow *window = NULL;

  window = current_window;
  if(!window)
    return;

  /* display error */
  gtk_image_set_from_stock(GTK_IMAGE(window->image),GTK_STOCK_DIALOG_ERROR,GTK_ICON_SIZE_DIALOG);
  gj_gtk_set_label_with_args(window->label_desc1, "<b>%s</b>", _("Failed to connect."));
  gj_gtk_set_label_with_args(window->label_desc2, (gchar*)gj_connection_connect_translate_status(event_id));
 
  gj_gtk_cp_complete(FALSE);
}

static void gj_gtk_cp_complete(gboolean success)
{
  GjConnectionProgressWindow *window = NULL;
  GjConnectionProgressCallback callback = NULL;

  window = current_window;
  if(!window)
    return;

  /* call callback */
  if(success == TRUE)
    {
      if((callback = window->cb_connected) != NULL)
	(callback)();

      /* close the window */
      gtk_widget_hide(window->window); 
    }
  else
    {
      gj_me_set_is_connected(FALSE);
      gj_me_set_uptime_stop();
      
      g_source_remove(window->timeout_id);
      window->timeout_id = -1;

      gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(window->progressbar),(gdouble)1/1);
      
      if((callback = window->cb_disconnected) != NULL)
	(callback)();
    }
}

static gboolean gj_gtk_cp_progress_timeout_cb(GjConnectionProgressWindow *window)
{
  if(!window)
    return FALSE;

  if(window->state == GjConnectionProgressStateAuthenticating)
    return FALSE;

  gtk_progress_bar_pulse(GTK_PROGRESS_BAR(window->progressbar));
  
  /* As this is a timeout function, return TRUE so that it
   * continues to get called */
  return TRUE;
} 

/*
 * GUI events
 */
static void on_button_action_clicked(GtkButton *button, GjConnectionProgressWindow *window)
{
  GjConnection c = NULL;

  if(!window)
    return;

  switch(window->state)
    {
    case GjConnectionProgressStateConnecting:
      c = gj_stream_get_connection(window->stream);

      gj_stream_set_is_disconnect_planned(window->stream, TRUE);
      gj_connection_disconnect(c);
      break;

    case GjConnectionProgressStateAuthenticationMethodRequested:
    case GjConnectionProgressStateAuthenticating:
    case GjConnectionProgressStateReady:
      gj_stream_close(window->stream);
      break;
    }
}

static void on_realize(GtkWidget *widget, GjConnectionProgressWindow *window)
{
  if(!window)
    return;

  /* if we do it on load, the window is hidden, and we get warnings */
  gdk_window_set_decorations (GDK_WINDOW(window->window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE | GDK_DECOR_MENU);
}

static void on_destroy(GtkWidget *widget, GjConnectionProgressWindow *window)
{
  current_window = NULL;
  if(!window)
    return;

  if(window->timeout_id != -1)
    {
      g_source_remove(window->timeout_id);
      window->timeout_id = -1;
    }

  if(window->server != NULL && window->port > 0)
    {
/*       GjStream stm = NULL; */

/*       if((stm = gj_stream_find_by_to(window->server)) != NULL) */
/* 	gj_stream_free(stm); */
    }

  if(window->server != NULL)
    g_free(window->server);
  if(window->username != NULL)
    g_free(window->username);
  if(window->password != NULL)
    g_free(window->password);
  if(window->resource != NULL)
    g_free(window->resource);

  g_free(window);
}

static gboolean on_delete_event(GtkWidget *widget, GdkEvent *event,  GjConnectionProgressWindow *window)
{
  /* return TRUE so it doesnt close */
  return TRUE;
}



#ifdef __cplusplus
}
#endif
 
