/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_debug.c,v $
 *
 * $Author: mr $
 * $Date: 2004/02/01 13:23:04 $
 * 
 * $Revision: 1.2 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "gj_main.h"
#include "gj_parser.h"

#include "gj_gtk_debug.h"
#include "gj_gtk_support.h"


struct 
{
  GtkWidget *window;

  GtkWidget *textview_input;
  GtkWidget *textview_output;

  GtkWidget *button_input;
  GtkWidget *button_output;
  
} ctlDebug;


/*
 * GUI functions
 */
static void on_button_debug_input_clicked(GtkButton *button, gpointer user_data);
static void on_button_debug_output_clicked(GtkButton *button, gpointer user_data);

static gboolean on_debug_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);

/*
 * code
 */
gboolean gj_gtk_debug_load()
{
  GladeXML *xml = NULL;

  if(ctlDebug.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(ctlDebug.window));
      return TRUE;
    }

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "debug",
			      NULL,
			      "debug", &ctlDebug.window,
			      "textview_input", &ctlDebug.textview_input,
			      "textview_output", &ctlDebug.textview_output,
			      "button_input", &ctlDebug.button_input,
			      "button_output", &ctlDebug.button_output,
			      NULL);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "debug", "delete_event", on_debug_delete_event,
		       "button_input", "clicked", on_button_debug_input_clicked,
		       "button_output", "clicked", on_button_debug_output_clicked,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  gdk_window_set_decorations (GDK_WINDOW(ctlDebug.window->window), 
			      GDK_DECOR_ALL);

  return TRUE;
}

gboolean gj_gtk_debug_unload()
{
  if(ctlDebug.window == NULL)
    return TRUE;

  gtk_widget_destroy(ctlDebug.window);
  ctlDebug.window = NULL;

  return TRUE;
}


/*
 * GUI events
 */
static void on_button_debug_input_clicked(GtkButton *button, gpointer user_data)
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter_start;
  GtkTextIter iter_end; 

  gchar *text = NULL;

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlDebug.textview_input));
  gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end, TRUE);

  g_message("on_button_debug_input_clicked: inputing buffer to XML parser: '%s'", text);

  if(g_utf8_validate(text, -1, NULL) == FALSE)
    g_warning("%s: text is not valid utf8", __FUNCTION__);
    
  gj_parser_push_input_from_connection(NULL, text);

  /* clean up */
  g_free(text);
}

static void on_button_debug_output_clicked(GtkButton *button, gpointer user_data)
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter_start;
  GtkTextIter iter_end; 

  gchar *text = NULL;
    
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlDebug.textview_output));
  gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end, TRUE);

  g_message("on_button_debug_input_clicked: outputing buffer to server: '%s'", text);

  gj_parser_push_output_from_text(NULL, text);

  /* clean up */
  g_free(text);
}

static gboolean on_debug_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  gj_gtk_debug_unload();
  return TRUE;
}

#ifdef __cplusplus
}
#endif
