/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_headline.c,v $
 *
 * $Author: mr $
 * $Date: 2004/02/01 13:23:05 $
 * 
 * $Revision: 1.3 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#ifndef G_OS_WIN32
#include <libgnome/gnome-i18n.h>
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gj_main.h"
#include "gj_roster.h"
#include "gj_roster_item.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_translate.h"
#include "gj_event.h"
#include "gj_support.h"
#include "gj_config.h"
#include "gj_jid.h"

#include "gj_gtk_roster.h"
#include "gj_gtk_headline.h" 
#include "gj_gtk_preferences.h"
#include "gj_gtk_support.h"
#include "gj_gtk_stock.h"


struct t_controls
{
  GtkWidget *window;

  GtkWidget *textview;

  GtkWidget *button_clear;
  GtkWidget *button_close;

};

struct t_HeadlineWindow
{
  gint ri_id;

  /*   gint x; */
  /*   gint y; */
  /*   gint w; */
  /*   gint h; */

  struct t_controls controls;
};


GList *headline_window_list = NULL;


/* list functions */
static gboolean gj_gtk_hw_list_add(HeadlineWindow hw);
static GList *gj_gtk_hw_list_get();

/* item functions */
static HeadlineWindow gj_gtk_hw_new(GjRosterItem ri);
#if 0
static gboolean gj_gtk_hw_free(HeadlineWindow hw);
#endif

/* find functions */
static HeadlineWindow gj_gtk_hw_find_by_ri_id(gint id);
static HeadlineWindow gj_gtk_hw_find_by_widget(GtkWidget *widget);

/*
 * member functions
 */

/* sets */

/* gets */
static gint gj_gtk_hw_get_ri_id(HeadlineWindow hw);

/* 
 * window functions 
 */
static gboolean gj_gtk_hw_load(HeadlineWindow hw);

static gboolean gj_gtk_hw_hide(HeadlineWindow hw);
static gboolean gj_gtk_hw_hide_by_widget(GtkWidget *widget);

static void gj_gtk_hw_raise();

static gboolean gj_gtk_hw_setup(HeadlineWindow hw);

static gboolean gj_gtk_hw_add(HeadlineWindow hw, GjMessage m);
static gboolean gj_gtk_hw_clear(HeadlineWindow hw);

/* 
 * gui callbacks 
 */
static void on_textbuffer_hw_changed(GtkTextBuffer *textbuffer, gpointer user_data);
static gboolean on_textview_hw_motion_notify_event(GtkWidget *widget, GdkEventMotion *event, gpointer user_data);
static gboolean on_textview_hw_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data);

static void on_button_hw_clear_clicked(GtkButton *button, gpointer user_data);
static void on_button_hw_close_clicked(GtkButton *button, gpointer user_data);

static gboolean on_headline_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);

/*
 * list functions
 */
static gboolean gj_gtk_hw_list_add(HeadlineWindow hw)
{
  if(hw == NULL)
    {
      g_warning("%s: headline window was NULL", __FUNCTION__);
      return FALSE;
    }

  headline_window_list = g_list_append(headline_window_list,hw);
  return TRUE;
}

static GList *gj_gtk_hw_list_get()
{
  return headline_window_list;
}

/*
 * item functions
 */
static HeadlineWindow gj_gtk_hw_new(GjRosterItem ri)
{
  HeadlineWindow hw = NULL;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__);
      return NULL;
    }

  if((hw = (HeadlineWindow) g_new(struct t_HeadlineWindow,1)) != NULL)
    {
      hw->ri_id = gj_roster_item_get_id(ri);

      hw->controls.window = NULL;
      hw->controls.textview = NULL;
      hw->controls.button_clear = NULL;
      hw->controls.button_close = NULL;

      gj_gtk_hw_list_add(hw);
    }

  return hw;
}

#if 0
static gboolean gj_gtk_hw_free(HeadlineWindow hw)
{
  if(hw == NULL)
    {
      g_warning("%s: headline window was NULL", __FUNCTION__);
      return FALSE;
    }

  /* free from list */
  gj_gtk_hw_list_del(hw);
  
  /* destroy members */
  gj_gtk_hw_destroy(hw);

  /* free struct */
  g_free(hw);
  
  return TRUE;
}
#endif

/*
 * find functions
 */
static HeadlineWindow gj_gtk_hw_find_by_ri_id(gint id)
{
  gint index = 0;

  if(id < 0)
    {
      g_warning("%s: id:%d was < 0", __FUNCTION__, id);
      return NULL;
    }

  for(index=0;index<g_list_length(gj_gtk_hw_list_get());index++)
    {
      HeadlineWindow hw = g_list_nth_data(gj_gtk_hw_list_get(), index);

      if(hw == NULL) continue;
      if(hw->ri_id == id) return hw;
    }

  return NULL;  
}

static HeadlineWindow gj_gtk_hw_find_by_widget(GtkWidget *widget)
{
  gint index = 0;
  GtkWidget *toplevel = NULL;

  if(widget == NULL)
    {
      g_warning("%s: widget was NULL", __FUNCTION__);
      return NULL;
    }

  /* get top level widget */
  toplevel = gtk_widget_get_toplevel(widget);

  if(toplevel == NULL)
    {
      g_warning("%s: top level widget was NULL", __FUNCTION__);
      return NULL;
    }

  for(index=0;index<g_list_length(gj_gtk_hw_list_get());index++)
    {
      HeadlineWindow hw = g_list_nth_data(gj_gtk_hw_list_get(), index);

      if(hw == NULL) continue;
      if(hw->controls.window == toplevel) return hw;
    }

  return NULL;  
}

/*
 * member functions 
 */

/* sets */

/* gets */
static gint gj_gtk_hw_get_ri_id(HeadlineWindow hw)
{
  if(hw == NULL)
    {
      g_warning("%s: headline window was NULL", __FUNCTION__);
      return -1;
    }

  return hw->ri_id;
}

/*
 * window functions
 */
gboolean gj_gtk_hw_show(HeadlineWindow hw, gboolean show)
{
  if(hw == NULL)
    {
      g_warning("%s: headline window was NULL", __FUNCTION__);
      return FALSE;
    }
  
  if(show == TRUE)
    gtk_widget_show(hw->controls.window);
  else
    gtk_widget_hide(hw->controls.window);

  return TRUE;
}

static void gj_gtk_hw_raise(HeadlineWindow hw)
{
  if(hw == NULL)
    {
      g_warning("%s: headline window was NULL", __FUNCTION__);
      return;
    }
  
  if(hw->controls.window == NULL)
    {
      g_warning("%s: window was NULL", __FUNCTION__);
      return;
    }
  
  gtk_window_present(GTK_WINDOW(hw->controls.window));
}

static gboolean gj_gtk_hw_load(HeadlineWindow hw)
{
  GladeXML *xml = NULL;
  GdkPixbuf *pb = NULL;
  
  if(hw == NULL)
    {
      g_warning("%s: headline window was NULL", __FUNCTION__);
      return FALSE;
    }

  if(hw->controls.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(hw->controls.window));
      return TRUE;
    }


  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "headline",
			      NULL,
			      "headline", &hw->controls.window,
			      "textview", &hw->controls.textview,
			      "button_clear", &hw->controls.button_clear,
			      "button_close", &hw->controls.button_close,
			      NULL);

  gtk_widget_set_events(GTK_WIDGET(hw->controls.textview),
			GDK_EXPOSURE_MASK | 
			GDK_POINTER_MOTION_MASK | 
			GDK_POINTER_MOTION_HINT_MASK | 
			GDK_BUTTON_RELEASE_MASK | 
			GDK_STRUCTURE_MASK | 
			GDK_PROPERTY_CHANGE_MASK);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "headline", "delete_event", on_headline_delete_event,
		       "button_clear", "clicked", on_button_hw_clear_clicked,
		       "button_close", "clicked", on_button_hw_close_clicked,
		       "textview", "button_press_event", on_textview_hw_button_press_event,
		       "textview", "motion_notify_event", on_textview_hw_motion_notify_event,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  gdk_window_set_decorations (GDK_WINDOW(hw->controls.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE | GDK_DECOR_MENU);

  /* set icon */
  pb = gj_gtk_pixbuf_new_from_stock(GJ_STOCK_MESSAGE_HEADLINE);
  gtk_window_set_icon(GTK_WINDOW(hw->controls.window), pb);
  g_object_unref(G_OBJECT(pb));

  /* set up model */
  gj_gtk_hw_setup(hw);

  return TRUE;
}

gboolean gj_gtk_hw_load_by_ri(GjRosterItem ri)
{
  HeadlineWindow hw = NULL;
  GjJID j = NULL;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__);
      return FALSE;
    }

  j = gj_roster_item_get_jid(ri);

  if((hw = gj_gtk_hw_find_by_ri_id(gj_roster_item_get_id(ri))) == NULL)
    {
      g_message("%s: could not get roster item by id:%d (jid:'%s')",
		__FUNCTION__, gj_roster_item_get_id(ri), gj_jid_get_full(j));
      return FALSE;     
    }

  gj_gtk_hw_load(hw);
  return TRUE;    
}

static gboolean gj_gtk_hw_hide(HeadlineWindow hw)
{
  if(hw == NULL)
    {
      g_warning("%s: headline window was NULL", __FUNCTION__);
      return FALSE;
    }


  gtk_widget_hide(GTK_WIDGET(hw->controls.window));
  g_message("%s: hidden...", __FUNCTION__);
  
  return TRUE;
}

static gboolean gj_gtk_hw_hide_by_widget(GtkWidget *widget)
{
  HeadlineWindow hw = NULL;

  if(widget == NULL)
    {
      g_warning("%s: widget was NULL", __FUNCTION__);
      return FALSE;
    }

  if((hw = gj_gtk_hw_find_by_widget(widget)) == NULL)
    {
      g_warning("%s: could not find headline window from widget", __FUNCTION__);
      return FALSE;
    }

  gj_gtk_hw_hide(hw);
  return TRUE;
}

static gboolean gj_gtk_hw_setup(HeadlineWindow hw)
{
  GtkTextBuffer *buffer = NULL;
  gchar *title = NULL;
  
  GjRosterItem ri = NULL;
  GjJID j = NULL;

  if(hw == NULL)
    {
      g_warning("%s: headline window was NULL", __FUNCTION__);
      return FALSE;
    }

  if((ri = gj_rosters_find_item_by_id(gj_gtk_hw_get_ri_id(hw))) == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__);
      return FALSE;
    }

  j = gj_roster_item_get_jid(ri);

  if((buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(hw->controls.textview))) == NULL)
    {
      g_warning("%s: buffer was NULL", __FUNCTION__);
      return FALSE;
    }
  
  /* set function callbacks */
  g_signal_connect(G_OBJECT(buffer), "changed", 
		   G_CALLBACK(on_textbuffer_hw_changed), 
		   NULL);

  /* set styles */
  gtk_text_buffer_create_tag(buffer, "bold", 
			     "weight", PANGO_WEIGHT_BOLD, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "italic", 
			     "style", PANGO_STYLE_ITALIC, 
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "blue_foreground", 
			     "foreground", "darkblue", 
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "red_foreground", 
			     "foreground", "darkred", 
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "black_foreground", 
			     "foreground", "black", 
			     NULL); 
  gtk_text_buffer_create_tag(buffer, "url", 
			     "foreground", "blue", 
			     NULL); 
  gtk_text_buffer_create_tag(buffer, "url_desc", 
			     /* 			     "weight", PANGO_WEIGHT_BOLD,  */
			     "underline", PANGO_UNDERLINE_SINGLE,
			     NULL); 
  gtk_text_buffer_create_tag(buffer, "left_margin_small",
			     "left_margin", 5,
			     NULL);
  gtk_text_buffer_create_tag(buffer, "left_margin_large",
			     "left_margin", 30,
			     NULL);
  gtk_text_buffer_create_tag(buffer, "not_editable",
			     "editable", FALSE, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "underline",
			     "underline", PANGO_UNDERLINE_SINGLE, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "tiny_gap_after_line",
			     "pixels_below_lines", 5, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "small_gap_after_line",
			     "pixels_below_lines", 15, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "big_gap_after_line",
			     "pixels_below_lines", 30, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "big_gap_before_line",
			     "pixels_above_lines", 30, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "large",
			     "scale", PANGO_SCALE_LARGE, 
			     NULL);
  
  /* set title */
  title = g_strdup_printf("%s [%s]", _("Headlines for"), gj_jid_get_full(j));
  gtk_window_set_title(GTK_WINDOW(hw->controls.window), title);

  /* clean up */
  if(title != NULL) g_free(title);

  return TRUE;
}

static gboolean gj_gtk_hw_add(HeadlineWindow hw, GjMessage m)
{
  GtkTextView *textview = NULL;
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter;

  const gchar *time = NULL;
  const gchar *message = NULL;
    
  time = gj_get_time_and_date_from_timet(gj_message_get_time(m));
  message = gj_message_get_body(m);

  if(hw == NULL)
    {
      g_warning("%s: headline window was NULL", __FUNCTION__);
      return FALSE;
    }

  if((textview = GTK_TEXT_VIEW(hw->controls.textview)) == NULL)
    {
      g_warning("%s: textview was NULL", __FUNCTION__);
      return FALSE;
    }      
    
  if((buffer = gtk_text_view_get_buffer(textview)) == NULL)
    {
      g_warning("%s: buffer was NULL", __FUNCTION__);
      return FALSE;
    }    

  if(time == NULL)
    {
      g_warning("%s: time was NULL", __FUNCTION__);
      return FALSE;
    }

  if(message == NULL)
    {
      g_warning("%s: message was NULL", __FUNCTION__);
      return FALSE;
    }

  gtk_text_buffer_get_end_iter(buffer, &iter);

  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					   time, -1,
					   "bold",
					   "black_foreground",
					   "left_margin_small",
					   "tiny_gap_after_line",
					   NULL);

  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					   "\n", -1,
					   "black_foreground",
					   NULL);

  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					   gj_message_get_body(m), -1,
					   "red_foreground",
					   "left_margin_large",
					   NULL);

  if(gj_message_get_x_oob_url(m) != NULL && gj_message_get_x_oob_description(m) != NULL)
    {
      /* show at end of m */
      gtk_text_buffer_remove_all_tags(buffer, &iter, &iter);
      gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					       "\n", -1,
					       "black_foreground",
					       "left_margin_large",
					       NULL);

      /* show url */      
      gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					       gj_message_get_x_oob_description(m), -1,
					       "left_margin_large",
					       "url_desc",
					       NULL);

      gtk_text_buffer_remove_all_tags(buffer, &iter, &iter);
      gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					       "\n", -1,
					       "black_foreground",
					       NULL);

      gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					       gj_message_get_x_oob_url(m), -1,
 					       "left_margin_large",
					       "url",
					       NULL);
    }

  gtk_text_buffer_remove_all_tags(buffer, &iter, &iter);
  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, 
					   "\n\n\n", -1,
					   "black_foreground",
					   NULL);

  return TRUE;
}

static gboolean gj_gtk_hw_clear(HeadlineWindow hw)
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter_start;
  GtkTextIter iter_end;

  if(hw == NULL)
    {
      g_warning("%s: headline window was NULL", __FUNCTION__);
      return FALSE;
    }

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(hw->controls.textview));
  gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  
  /* clear buffer */
  gtk_text_buffer_delete(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  gtk_text_buffer_delete_mark_by_name(GTK_TEXT_BUFFER(buffer), "url");

  return TRUE;
}

HeadlineWindow gj_gtk_hw_notification_new(gint event_id, GjMessage m)
{
  HeadlineWindow hw = NULL;
  GjEvent ev = NULL;
  GjRosterItem ri = NULL;
  GjJID j = NULL;

  if(m == NULL)
    {
      g_warning("%s: message was NULL", __FUNCTION__);
      return NULL;
    }

  if((ev = gj_event_find_by_id(event_id)) == NULL)
    {
      g_warning("%s: event was NULL", __FUNCTION__);
      return NULL;
    }

  if((ri = gj_rosters_find_item_by_id(gj_event_get_ri_id(ev))) == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__);
      return NULL;
    }

  j = gj_roster_item_get_jid(ri);

  if((hw = gj_gtk_hw_find_by_ri_id(gj_roster_item_get_id(ri))) == NULL)
    {
      if((hw = gj_gtk_hw_new(ri)) == NULL)
	{
	  g_warning("%s: could not create new hw from jid:'%s'", __FUNCTION__, gj_jid_get_full(j));
	  return NULL;
	}

      gj_gtk_hw_load(hw);
    }
  else
    {
      /* should we show it? */
      if(gj_config_get_mpf_raise_window_on_message() == TRUE)
	gj_gtk_hw_raise(hw);
    }

  gj_gtk_hw_add(hw, m);

  return hw;
}

/*
 * GUI events
 */
static void on_textbuffer_hw_changed(GtkTextBuffer *textbuffer, gpointer user_data)
{
  gj_gtk_textbuffer_handle_changed(textbuffer, user_data);
}

static gboolean on_textview_hw_motion_notify_event(GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
{
  return gj_gtk_textview_handle_motion_notify_event(widget, event, user_data);
}

static gboolean on_textview_hw_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
  return gj_gtk_textview_handle_button_press_event(widget, event, user_data);
}

static void on_button_hw_clear_clicked(GtkButton *button, gpointer user_data)
{
  HeadlineWindow hw = NULL;
  
  if((hw = gj_gtk_hw_find_by_widget(GTK_WIDGET(button))) == NULL)
    {
      g_warning("%s: could not find headline window from button widget", __FUNCTION__);
      return;
    }
 
  gj_gtk_hw_clear(hw);
}

static void on_button_hw_close_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_hw_hide_by_widget(GTK_WIDGET(button));
}

static gboolean on_headline_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  gj_gtk_hw_hide_by_widget(GTK_WIDGET(widget));

  return TRUE;
}


#ifdef __cplusplus
}
#endif
