/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_iq_requests.h,v $
 *
 * $Author: mr $
 * $Date: 2004/02/01 13:23:06 $
 * 
 * $Revision: 1.8 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#ifndef __gj_iq_requests_h
#define __gj_iq_requests_h

#include "gj_typedefs.h"


gboolean gj_iq_request_greeting(const gchar *to, 
				guint port, 
				GjStreamCallback cb_connected, 
				GjStreamCallback cb_disconnected, 
				GjStreamCallback cb_error,
				void *userdata);

gboolean gj_iq_request_goodbye(const gchar *to);

gboolean gj_iq_request_auth_method(const gchar *username, 
				   GjInfoqueryCallback cb,
				   void *userdata);

gboolean gj_iq_request_auth_with_plain_password(const gchar *username, 
						const gchar *password, 
						const gchar *resource, 
						GjInfoqueryCallback cb,
						void *userdata);

gboolean gj_iq_request_auth_with_digest_password(const gchar *username, 
						 const gchar *password, 
						 const gchar *resource, 
						 GjInfoqueryCallback cb,
						 void *userdata);


gboolean gj_iq_request_roster(GjInfoqueryCallback cb,
			      void *userdata);

gboolean gj_iq_request_roster_remove_contact(const gchar *jid, 
					     GjInfoqueryCallback cb,
					     void *userdata);

gboolean gj_iq_request_roster_add_contact(const gchar *jid, 
					  const gchar *name, 
					  GList *groups, 
					  GjInfoqueryCallback cb,
					  void *userdata);

gboolean gj_iq_request_vcard(const gchar *to, 
			     GjInfoqueryCallback cb, 
			     void *userdata);

gboolean gj_iq_request_version(const gchar *to, 
			       GjInfoqueryCallback cb, 
			       void *userdata);

gboolean gj_iq_request_time(const gchar *to, 
			    GjInfoqueryCallback cb, 
			    void *userdata);

gboolean gj_iq_request_last(const gchar *to, 
			    GjInfoqueryCallback cb, 
			    void *userdata);

gboolean gj_iq_request_password_change(const gchar *to, 
				       const gchar *password, 
				       GjInfoqueryCallback cb,
				       void *userdata);

gboolean gj_iq_request_user_change(const gchar *jid, 
				   const gchar *name, 
				   GList *group, 
				   GjInfoqueryCallback cb,
				   void *userdata);

gboolean gj_iq_request_vcard_change(const gchar *fn,
				    const gchar *given,
				    const gchar *family,
				    const gchar *nickname,
				    const gchar *url,
				    const gchar *tel,
				    const gchar *email,
				    const gchar *desc,
				    const gchar *bday,
				    const gchar *street,
				    const gchar *extadd,
				    const gchar *locality,
				    const gchar *region,
				    const gchar *pcode,
				    const gchar *country,
				    const gchar *orgname,
				    const gchar *orgunit,
				    const gchar *title,
				    const gchar *role,
				    GjInfoqueryCallback cb,
				    void *userdata);

gboolean gj_iq_request_register_user(const gchar *to, 
				     const gchar *username, 
				     const gchar *password, 
				     GjInfoqueryCallback cb,
				     void *userdata);

#if 0
gboolean gj_iq_request_register(const gchar *to, 
				GjInfoqueryCallback cb,
				void *userdata);
#endif

gboolean gj_iq_request_register_requirements(const gchar *to, 
					     GjInfoqueryCallback cb,
					     void *userdata);

gboolean gj_iq_request_register(const gchar *to, 
				GjInfoqueryCallback cb, 
				const gchar *key,
				const gchar *username,
				const gchar *password,
				const gchar *email,
				const gchar *nickname,
				void *userdata);

gboolean gj_iq_request_unregister(const gchar *to, 
				  const gchar *key,
				  GjInfoqueryCallback cb,
				  void *userdata);


gboolean gj_iq_request_agents(const gchar *server, 
			      GjInfoqueryCallback cb,
			      void *userdata);

gboolean gj_iq_request_browse(const gchar *server, 
			      GjInfoqueryCallback cb,
			      void *userdata);

gboolean gj_iq_request_search_requirements(const gchar *to, 
					   GjInfoqueryCallback cb,
					   void *userdata);

gboolean gj_iq_request_search(const gchar *to, 
			      GjInfoqueryCallback cb, 
			      void *userdata,
			      ...);


gboolean gj_iq_request_file_transfer_si_set(const gchar *jid, 
					    const gchar *sid,
					    const gchar *mime_type,
					    const gchar *file_name,
					    const gchar *file_size,
					    GjInfoqueryCallback cb,
					    void *userdata);

gboolean gj_iq_request_file_transfer_si_result(const gchar *qid, 
					       const gchar *jid, 
					       const gchar *sid,
					       const gchar *mime_type,
					       const gchar *file_name,
					       const gchar *file_size,
					       GjInfoqueryCallback cb,
					       void *userdata);

gboolean gj_iq_request_file_transfer_bytestreams_result(const gchar *qid,
							const gchar *jid, 
							const gchar *sid,
							const gchar *stream_jid_used,
							GjInfoqueryCallback cb,
							void *userdata);

gboolean gj_iq_request_file_transfer_ibb_init_send(const gchar *jid, 
						   gint block_size,
						   GjInfoqueryCallback cb,
						   void *userdata);


gboolean gj_iq_request_disco_info(const gchar *jid, 
				  GjInfoqueryCallback cb,
				  void *userdata);

gboolean gj_iq_request_disco_items(const gchar *jid, 
				   GjInfoqueryCallback cb,
				   void *userdata);

#endif

#ifdef __cplusplus
}
#endif
