/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_me.c,v $
 *
 * $Author: mr $
 * $Date: 2004/02/04 13:25:07 $
 * 
 * $Revision: 1.3 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "gj_main.h"
#include "gj_presence.h"

#include "gj_me.h"


static GjMeUptimeCallback gj_me_get_uptime_cb();
static gboolean gj_me_uptime_timeout(gpointer userdata);


/* connection */
static gboolean is_connected = FALSE;
  
static gchar *connected_host = NULL;
static guint connected_port = -1;
static gchar *connected_jid = NULL;
  
/* auto away */
static gboolean is_auto_away = FALSE;
static GjPresenceShow auto_away_presence_show = GjPresenceShowNormal;  /* either xa or away */
  
/* presence */
static GjPresenceType presence_type = GjPresenceTypeAvailable;
static GjPresenceShow presence_show = GjPresenceShowNormal;
static gchar *presence_status = NULL;
static gint8 presence_priority = -1;

/* last namespace */
static GTimer *uptime = NULL;
static GjMeUptimeCallback uptime_cb = NULL;
  

/* get members */
gboolean gj_me_get_is_connected()
{
  return is_connected;
}

const gchar *gj_me_get_connected_host()
{
  return connected_host;
}

guint gj_me_get_connected_port()
{
  return connected_port;
}

gboolean gj_me_get_is_auto_away()
{
  return is_auto_away;
}

GjPresenceShow gj_me_get_auto_away_presence_show() 
{
  return auto_away_presence_show;
}

const gchar *gj_me_get_connected_jid() 
{
  return connected_jid;
}

gchar *gj_me_get_connected_jid_without_resource()
{
  gchar **vjid = NULL;
  gchar *jid = NULL;

  if(jid == NULL)
    return NULL;

  vjid = g_strsplit(jid, "/", -1);
  
  if(vjid == NULL || vjid[0] == NULL)
    jid = g_strdup(jid);
  else
    jid = g_strdup(vjid[0]);

  if(vjid != NULL) g_strfreev(vjid);
  return jid;
}

GjPresenceType gj_me_get_presence_type()
{
  return presence_type;
}

GjPresenceShow gj_me_get_presence_show()
{
  return presence_show;
}

const gchar *gj_me_get_presence_status()
{
  return presence_status;
}

gint8 gj_me_get_presence_priority()
{
  return presence_priority;
}

gdouble gj_me_get_uptime()
{
  gdouble seconds = 0;
  gulong microseconds = 0;

  if(uptime == NULL)
    return 0;

  seconds = g_timer_elapsed(uptime,&microseconds);

  return seconds;
}

GjMeUptimeCallback gj_me_get_uptime_cb()
{
  return uptime_cb;
}

/* set members */
void gj_me_set_is_connected(const gboolean connected)
{
  is_connected = connected;
}

gboolean gj_me_set_connected_host(const gchar *host)
{
  if(host == NULL)
    {
      g_warning("%s: host is NULL", __FUNCTION__);
      return FALSE;
    }

  if(connected_host != NULL)
    {
      g_free(connected_host);
      connected_host = NULL;
    }

  connected_host = g_strdup(host);
  return TRUE;
}

gboolean gj_me_set_connected_port(const guint port)
{
  if(port < 1 || port > 65535)
    {
      g_warning("%s: port:%d was invalid", __FUNCTION__, port);
      return FALSE;
    }

  connected_port = port;
  return TRUE;
}

void gj_me_set_is_auto_away(const gboolean auto_away)
{
  is_auto_away = auto_away;
}

gboolean gj_me_set_connected_jid(const gchar *jid)
{
  if(jid == NULL)
    {
      g_warning("%s: jid is NULL", __FUNCTION__);
      return FALSE;
    }

  if(connected_jid != NULL)
    {
      g_free(connected_jid);
      connected_jid = NULL;
    }

  connected_jid = g_strdup(jid);
  return TRUE;
}

void gj_me_set_auto_away_presence_show(const GjPresenceShow show) 
{
  auto_away_presence_show = show;
}

void gj_me_set_presence_type(const GjPresenceType type)
{
  presence_type = type;
}

void gj_me_set_presence_show(const GjPresenceShow show)
{
  presence_show = show;
}

void gj_me_set_presence_status(const gchar *status)
{
  if(presence_status != NULL)
  {
      g_free(presence_status);
      presence_status = NULL;
  }

  if(status)
      presence_status = g_strdup(status);
}

void gj_me_set_presence_priority(const gint8 priority)
{
  presence_priority = priority;
}

void gj_me_set_uptime_start()
{
  gj_me_set_uptime_stop();

  uptime = g_timer_new();
  g_timer_start(uptime); 

  g_timeout_add(1000, gj_me_uptime_timeout, NULL);
}

void gj_me_set_uptime_stop()
{
  if(uptime != NULL)
    {
      g_timer_stop(uptime);
      g_timer_destroy(uptime);
      uptime = NULL;
    }
}

void gj_me_set_uptime_restart()
{
  gj_me_set_uptime_stop();
  gj_me_set_uptime_start();
}

gboolean gj_me_set_uptime_cb(GjMeUptimeCallback cb)
{
  if(cb == NULL)
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return FALSE;
    }
  
  uptime_cb = cb;
  return TRUE;
}

void gj_me_set_uptime_cb_to_null()
{
  uptime_cb = NULL;
}


gboolean gj_me_uptime_timeout(gpointer userdata)
{
  gdouble seconds = 0;
  GjMeUptimeCallback cb = NULL;

  /* we stop the timerout when the timer is freed */
  if(uptime == NULL)
    return FALSE;

  /* we continue to use the timer without the callback */
  if(uptime_cb == NULL)
    return TRUE;

  cb = gj_me_get_uptime_cb();
  seconds = gj_me_get_uptime();

  (cb)(seconds);

  return TRUE;
}


#ifdef __cplusplus
}
#endif
